@@ -11,9 +11,18 @@
 
 
 import abc
+import weakref
+import functools
 import pymongo
 
 
+try:
+    basestring
+except NameError:
+    # must by python3
+    basestring = (str, bytes)
+
+
 class WrongLevel(AttributeError):
     pass
 
@@ -74,6 +83,14 @@ class Collection(pymongo.collection.Collection):
             return model_instance
 
 
+def collectionmethod(fn):
+    @classmethod
+    @functools.wraps(fn)
+    def wrapper(model, *args, **kwargs):
+        return fn(model.collection, *args, **kwargs)
+    return wrapper
+
+
 class AttrDict(dict):
     """A dictionary whose keys are accessible with dot notation
     cool __setitem__ -> http://stackoverflow.com/a/2588648/1299695
@@ -83,8 +100,14 @@ class AttrDict(dict):
         self.update(*args, **kwargs)
 
     def __setitem__(self, key, value):
-        if isinstance(value, dict) and not isinstance(value, AttrDict):
-            value = AttrDict(value)
+        if isinstance(value, dict):
+            if not isinstance(value, AttrDict):
+                value = AttrDict(value)
+        elif not isinstance(value, basestring):
+            try:
+                value = [AttrDict(v) for v in value]
+            except TypeError:
+                pass
         super(AttrDict, self).__setitem__(key, value)
 
     def update(self, *args, **kwargs):
@@ -133,6 +156,8 @@ class AttrDict(dict):
 class Model(AttrDict):
     """Helper methods and properties."""
 
+    _live_documents = weakref.WeakValueDictionary()
+
     def __new__(cls, *args, **kwargs):
         """Return an instance of the class.
         ABCMeta isn't enforced because AttrDict screws around with __getattr__,
@@ -166,11 +191,15 @@ class Model(AttrDict):
         """Create or update the instance in the database. Returns the pymongo
         ObjectId. See :meth: pymongo.collection.Collection.save.
         """
-        return self.collection.save(self, *args, **kwargs)
+        _id = self.collection.save(self, *args, **kwargs)
+        self._live_documents[_id] = self
+        return _id
 
     def insert(self, *args, **kwargs):
         """Save as a new document in the database. Wraps collection.insert"""
-        return self.collection.insert(self, *args, **kwargs)
+        _id = self.collection.insert(self, *args, **kwargs)
+        self._live_documents[_id] = self
+        return _id
 
     def remove(self, spec=None, *args, **kwargs):
         """Remove this document from the databse."""
@@ -187,8 +216,11 @@ class Model(AttrDict):
     @classmethod
     def inflate(cls, json):
         """Return a model instance given its MongoDB json representation"""
-        instance = cls.__new__(cls)
-        Model.__init__(instance, json)
+        if '_id' in json and json['_id'] in cls._live_documents:
+            instance = cls._live_documents[json['_id']]
+        else:
+            instance = cls.__new__(cls)
+            Model.__init__(instance, json)
         return instance
 
     def __repr__(self):
